#include <GL/gl.h>
#include <GL/glu.h>
#include <SDL/SDL.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "map.h"
#include "rand.h"

#define RESX 1024
#define RESY 768
#define DEPTH 16

#define INTERFACEX 256
#define INTERFACEY 32

#define TILESIZE 32

SDL_Surface *screen;
SDL_Rect view;

static char * treepath = "data/graphics/terrain/trees/";

struct _terain_properties sdl_terrain_properties[] = {
    { MOUNTAIN,  "Mountains",     10,    5,   -10,  1, 1, "data/graphics/terrain/mountain.bmp" },
    { HILL,      "Hills",          5,    3,    -5,  1, 1, "data/graphics/terrain/hill.bmp" },
    { PLAIN,     "Plains",         0,    0,    15,  1, 1, "data/graphics/terrain/plain.bmp" },
    { SAND,      "Sand",           5,    5,    20,  1, 1, "data/graphics/terrain/sand.bmp" },
    { ROUGH,     "Rough Terrain",  5,    5,    10,  1, 1, "data/graphics/terrain/rough.bmp" },
    { ROAD,      NULL,             0,    0,    30,  1, 1, "data/graphics/terrain/road.bmp" },
    { RIVER,     "River",         -5,  -10,   -20,  0, 1, "data/graphics/terrain/river.bmp" },
    { LAKE,      "Lake",         -10,  -10,   -20,  0, 1, "data/graphics/terrain/lake.bmp" },
    { BUILDING,  NULL,             0,    0,     0,  0, 0, "data/graphics/terrain/building.bmp" },
    { -1,        NULL,             0,    0,     0,  0, 0, NULL } 
};

struct _structure_properties sdl_structure_properties[] = {
    { BARRACKS,        "Infantry Barracks",              2,   2,   0,   20,   1000,  1, "data/graphics/buildings/barracks.bmp" },
    { FACTORY,         "Vehicle Factory",                3,   3,   0,   50,   1500,  1, "data/graphics/buildings/factory.bmp" },
    { BASE,            "Command Center",                 2,   2,   50,   0,   3000,  1, "data/graphics/buildings/base.bmp"  },
    { POWERPLANT,      "A Nuclear Power Plant",          2,   2,   100,  0,   1000,  0, "data/graphics/buildings/powerplant.bmp" },
    { RESOURCE_CENTER, "Advanced Resource Conversion",   2,   2,   0,   20,   1000,  1, "data/graphics/buildings/resourcecenter.bmp" },
    { -1,              NULL,                             0,   0,   0,    0,      0,  0, NULL }
};


int sdl_render_trees(SDL_Surface *screen, map_properties * map);
int sdl_render_map(SDL_Surface *screen, map_properties * map, unsigned slocx, unsigned slocy);
int sdl_render_interface(SDL_Surface *screen);

int make_level (unsigned i, char * level)
{
    snprintf(level, 64, "%s%d%s", "data/level", i, ".dat");
    return 0;
}

int sdl_init_graphics()
{
    
    if (SDL_Init(SDL_INIT_VIDEO) != 0) 
	return 1;

    atexit(SDL_Quit);
    
    screen = SDL_SetVideoMode(RESX, RESY, DEPTH, SDL_HWSURFACE | SDL_HWACCEL | SDL_HWPALETTE | SDL_DOUBLEBUF);
    if (screen == NULL) 
	return 1;
    
    SDL_WM_SetCaption("TDune","TDune");
    
    return 0;
}

int sdl_game_loop () {
        
	SDL_Event event;
	map_properties * map;
	unsigned i = 1;
	unsigned slocx,slocy;
	char * level;
	
	slocx=slocy=0;
    
	initrandom();
	sdl_render_interface(screen);
	
	view.x = 0;
        view.y = 0;
	view.w = RESX - INTERFACEX;
	view.h = RESY - INTERFACEY;
    
        SDL_SetClipRect(screen, &view);
	
	level = (char *) malloc (64 * sizeof(char));
	make_level(i,level);
	
	map = map_load_map(level);
	//map_get_statistics(map);
	
	sdl_render_map(screen, map, slocx, slocy);

    SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY,SDL_DEFAULT_REPEAT_INTERVAL);
	
    while (SDL_WaitEvent(&event) != 0) {
	SDL_keysym keysym;
	initrandom();
	switch (event.type) {
	
	    case SDL_KEYDOWN:
		keysym = event.key.keysym;
		
		if (keysym.sym == SDLK_q)
			exit(0);
		
		if (keysym.sym == SDLK_n) {
		    if (i < 4) { 
			i++; 
			slocx=slocy=0;
			make_level(i, level);
			map = map_load_map(level);
			//map_get_statistics(map);
			sdl_render_map(screen, map, slocx, slocy);
		    }
		}
		
		if (keysym.sym == SDLK_p) {
		    if (i > 1) {
			i--;
			slocx=slocy=0;
			make_level(i, level);
			map = map_load_map(level);
			//map_get_statistics(map);
			sdl_render_map(screen, map,slocx, slocy);
		    }
		}
		
		if (keysym.sym == SDLK_RIGHT) {
			if (slocx < map->width) {
			    slocx++;
			    printf("slocx:%d\n",slocx);
			    sdl_render_map(screen, map, slocx, slocy);
			}
		    }
		
		if (keysym.sym == SDLK_LEFT) {
			if (slocx > 0) {
			    slocx--;
			    sdl_render_map(screen, map, slocx, slocy);
			}
		    }
		    
		if (keysym.sym == SDLK_UP) {
			if (slocy > 0) {
			    slocy--;
			    sdl_render_map(screen, map, slocx, slocy);
			}
		    }
				
		if (keysym.sym == SDLK_DOWN) {
			if (slocy < map->height) {
			    slocy++;
			    printf("slocy:%d\n",slocy);
			    sdl_render_map(screen, map, slocx, slocy);
			}
		    }
		    
		if (keysym.sym == SDLK_t) 
		    sdl_render_trees(screen, map);		

    		if (keysym.sym == SDLK_f) 
		    SDL_WM_ToggleFullScreen(screen);		
		
		break;		
	    case SDL_QUIT:
		exit(0);
	}
    }
    
    SDL_FreeSurface(screen);	
    return 0;
}

int sdl_render_interface(SDL_Surface *screen)
{
    SDL_Surface *temp;
    SDL_Rect src, dest;
    unsigned i;
    
    temp = SDL_LoadBMP("data/graphics/interface/title.bmp");
    if (temp == NULL) {
        printf ("Error loading interface title\n");
        return 1;
    }
    src.x = 0;
    src.y = 0;
    src.w = temp->w;
    src.h = temp->h;
    
    dest.x = RESX - INTERFACEX;
    dest.y = 0;
    dest.w = temp->w;
    dest.h = temp->h;
    
    SDL_BlitSurface(temp,&src,screen,&dest);
    SDL_FreeSurface(temp);
    
    for (i = 256; i < RESY ; i+=256) {
	temp = SDL_LoadBMP("data/graphics/interface/tiley.bmp");
	if (temp == NULL) {
    	    printf ("Error loading interface tile\n");
    	    return 1;
	}
	
	src.x = 0;
	src.y = 0;
	src.w = temp->w;
	src.h = temp->h;
    
	dest.x = RESX - INTERFACEX;
	dest.y = i;
	dest.w = temp->w;
	dest.h = temp->h;
    
	SDL_BlitSurface(temp,&src,screen,&dest);
	SDL_FreeSurface(temp);
    }

    for (i=0; i< RESX; i+=256) {
	temp = SDL_LoadBMP("data/graphics/interface/tilex.bmp");
	if (temp == NULL) {
    	    printf ("Error loading interface tile\n");
    	    return 1;
	}
	
	src.x = 0;
	src.y = 0;
	src.w = temp->w;
	src.h = temp->h;
    
	dest.x = i;
	dest.y = RESY - INTERFACEY;
	dest.w = temp->w;
	dest.h = temp->h;
    
	SDL_BlitSurface(temp,&src,screen,&dest);
	SDL_FreeSurface(temp);
    }         
    SDL_Flip (screen);
    return 0;
}

int sdl_render_map(SDL_Surface *screen, map_properties * map, unsigned slocx, unsigned slocy)
{
    SDL_Surface * temp, *building;
    SDL_Rect src, dest;    
    location_properties * location;
    unsigned i,j,x,y;
    
    
    for (i = slocy , y=0; i < map->height && i < slocy + ( RESY - INTERFACEY)/TILESIZE ; i++,y++)
	for (j = slocx, x=0; j < map->width && j < slocx + (RESX - INTERFACEX)/TILESIZE; j++,x++) {
	
	    location = map->map[i][j];
	    
	    if (location->structure == -1) {
		
		//printf ("Trying to load terrain: %s\n", sdl_terrain_properties[location->terrain].image);
		
		temp = SDL_LoadBMP(sdl_terrain_properties[location->terrain].image);
    
		if (temp == NULL) {
		    printf ("Error loading: %s\n",sdl_terrain_properties[location->terrain].image);
		    return 1;
		}
		
		src.x = 0;
		src.y = 0;
		src.w = temp->w;
		src.h = temp->h;
    
		dest.x = x * temp->w;
		dest.y = y * temp->h;
		dest.w = temp->w;
		dest.h = temp->h;
    
		SDL_BlitSurface(temp, &src, screen, &dest);
		SDL_FreeSurface(temp);
		continue;
	    }
	    
	    if (location->origin == 1) {
		unsigned l,k;
		location_properties *templocation;
		
		for (l = 0; l < sdl_structure_properties[location->structure].height; l++)
		    for (k = 0; k < sdl_structure_properties[location->structure].width; k++) {
		    
			templocation = map->map[i+l][j+k];
			temp = SDL_LoadBMP(sdl_terrain_properties[templocation->terrain].image);
    
			if (temp == NULL) {
			    printf ("Error loading: %s\n",sdl_terrain_properties[templocation->terrain].image);
			    return 1;
			}
		    src.x = 0;
		    src.y = 0;
		    src.w = temp->w;
		    src.h = temp->h;
    
		    dest.x = (x+k) * temp->w;
		    dest.y = (y+l) * temp->h;
		    dest.w = temp->w;
		    dest.h = temp->h;
    
		    SDL_BlitSurface(temp, &src, screen, &dest);
		    SDL_FreeSurface(temp);
		}
		
		//printf ("Trying to load building: %s\n", sdl_structure_properties[location->structure].image);
		building = SDL_LoadBMP (sdl_structure_properties[location->structure].image);
		
		if (building == NULL) {
		    printf ("Error loading: %s\n",sdl_structure_properties[location->structure].image);
		    return 1;
		}
		
		SDL_SetColorKey(building, SDL_SRCCOLORKEY | SDL_RLEACCEL, 
			    (Uint32) SDL_MapRGB(building->format, 191, 123, 199));
    
    		src.x = 0;
		src.y = 0;
		src.w = building->w;
		src.h = building->h;
    
		dest.x = x * TILESIZE;
		dest.y = y * TILESIZE;
		dest.w = building->w;
		dest.h = building->h;
    
		SDL_BlitSurface(building, &src, screen, &dest);
		SDL_FreeSurface(building);
        }
    }

    //SDL_UpdateRect(screen, 0, 0, 0, 0);
    SDL_Flip(screen);
    return 0;
}


int sdl_render_trees(SDL_Surface *screen, map_properties * map)
{
    SDL_Surface * temp;
    SDL_Rect src, dest;    
    location_properties * location;
    unsigned i,x,y, numtrees, idtree;
    char * tree;
    
    tree = (char *) malloc (128 * sizeof(char));
    numtrees = (map->height * map->width)/4;
    
    for (i = 0; i < numtrees; i++) {
	    	    
	    x = (int) getrandom() % map->width;
	    y = (int) (int) getrandom() % map->height;;
	    idtree = (int) (getrandom() % 4);
	    location = map->map[y][x];
	    snprintf(tree,128,"%s%s%d%s",treepath, "tree", idtree, ".bmp");
	    //printf ("Trying to load tree: %s at location %d, %d\n", tree, x,y);
	    
	    if (location->structure == -1 && location->tree == -1 && 
		location->terrain == PLAIN ) {
		//printf ("Loading tree: %s at location %d, %d on terrain: %d\n", tree, x,y, location->terrain);
		location->tree = idtree;
		temp = SDL_LoadBMP(tree);
		if (temp == NULL) {
		    printf ("Error loading: %s\n", tree);
		    return 1;
		}
		SDL_SetColorKey(temp, SDL_SRCCOLORKEY | SDL_RLEACCEL, 
			    (Uint32) SDL_MapRGB(temp->format, 115, 109, 181));
		
		src.x = 0;
		src.y = 0;
		src.w = temp->w;
		src.h = temp->h;
    
		dest.x = x * temp->w;
		dest.y = y * temp->h;
		dest.w = temp->w;
		dest.h = temp->h;
    
		SDL_BlitSurface(temp, &src, screen, &dest);
		SDL_FreeSurface(temp);
	}
    }
    
    //SDL_UpdateRect(screen, 0, 0, 0, 0);
    SDL_Flip(screen);
    return 0;
}
    
